'use client'

import { useRef } from 'react';
import Markdown from 'react-markdown';

import * as style from './markdown.css';
import TabGroup from '../tab_group';
import Copy from '../copy';
import Text from './text';
import { map_to_type } from './text';

const initial = 0;

export default function MarkdownContent({ text }: { text: string }) {
	/*
	 * Originally used 'useState', but switching back to the 'Rendered'
	 * tab from 'Raw' was expensive and caused a hiccup
	 *
	 * Now, keep them both loaded in the DOM and use display to switch
	 * which is being rendered. It's less "Reactive" or whatever, but
	 * it is simply faster ¯\_(ツ)_/¯
	 */
	const refs = [
		useRef<HTMLDivElement | null>(null),
		useRef<HTMLDivElement | null>(null),
	]

	function tab_callback(active: number) {
		for (const [index, ref] of refs.entries()) {
			ref.current!.style.display = index == active ? 'block' : 'none';
		}
	}

	return <>
		<TabGroup tabs={[ 'Rendered', 'Raw' ]} initial={initial} tab_callback={tab_callback} />
		{[
			<Markdown
				key={0}
				components={{
					'code': (props) => {
						const split = props.className?.split('-') ?? ['none'];
						return (
							<div className={style.code_bounds}>
								<Copy className={style.copy} text={props.children as string}></Copy>
								<Text language={split.length == 0 ? 'none' : map_to_type(split[split.length - 1])} text={props.children as string} line_numbers={false} />
							</div>
						);
					}
				}}
			>{text}</Markdown>,
			<Text key={1} language={"markdown"} text={text} line_numbers={true} />
		].map((tab, index) => <div key={index} ref={refs[index]} style={{ display: index == initial ? 'block' : 'none' }}>{tab}</div>)}
	</>;
}
