import * as style from './terminal.css';

const ansi_normal_colors = [
	"#000000",
	"#bc2232",
	"#2e9662",
	"#a55d14",
	"#2f5dab",
	"#694Caa",
	"#006080",
	"#a5a2af",
];

const ansi_bright_colors = [
	"#44444f",
	"#f22c40",
	"#39bb7a",
	"#d6781a",
	"#407ee7",
	"#8466ea",
	"#0082ad",
	"#FFFFFF",
];

function ansi(text: string) {
	const segments    = text.split(/(?=\x1B)/g);
	const spans       = [];
	const style: {
		color?:           string,
		backgroundColor?: string,
		fontWeight?:      string,
	} = {};
	for (const [index, segment] of segments.entries()) {
		const ansi_segment = segment.substring(0, segment.indexOf('m') + 1);
		const escape_codes = [
			...
			ansi_segment.matchAll(/\d+/g)
		].map(element => Number(element[0]));
		if (escape_codes.length == 0) {
			style.color           = undefined;
			style.backgroundColor = undefined;
			style.fontWeight      = undefined;
		}
		for (const code of escape_codes) {
			if (code == 0) {
				style.color           = undefined;
				style.backgroundColor = undefined;
				style.fontWeight      = undefined;
			} else if (code == 1) {
				style.fontWeight = 'bold';
			} else if (30 <= code && code <= 37) {
				style.color = ansi_normal_colors[code - 30];
			} else if (90 <= code && code <= 97) {
				style.color = ansi_bright_colors[code - 90];
			} else if (40 <= code && code <= 47) {
				style.backgroundColor = ansi_bright_colors[code - 40];
			} else if (100 <= code && code <= 107) {
				style.backgroundColor = ansi_bright_colors[code - 100];
			}
		}
		spans.push(<span key={index} style={{...style}}>{segment.substring(ansi_segment.length)}</span>);
	}
	return spans;
}

export default function Terminal({ text }: { text: string }) {
	return (
		<p className={style.content}>{ansi(text)}</p>
	);
}
