import { useEffect, useState } from 'react';
import { Highlight, themes } from 'prism-react-renderer';

import * as style from './text.css';

const type_map = {
	//          'markup',
	js:         'jsx',
	mjs:        'jsx',
	jsx:        'jsx',
	javascript: 'javascript',
	//          'js-extras',
	ts:         'tsx',
	tsx:        'tsx',
	typescript: 'typescript',
	swift:      'swift',
	kt:         'kotlin',
	kts:        'kotlin',
	ktm:        'kotlin',
	kotlin:     'kotlin',
	//          'objectivec',
	//          'reason',
	rs:         'rust',
	rust:       'rust',
	//          'graphql',
	yml:        'yaml',
	yaml:       'yaml',
	go:         'go',
	c:          'cpp',
	cpp:        'cpp',
	cxx:        'cpp',
	'c++':      'cpp',
	h:          'cpp',
	hpp:        'cpp',
	hxx:        'cpp',
	py:         'python',
	python:     'python',
	json:       'json',
	md:         'markdown',
	markdown:   'markdown',
	none:       'none',
}

export function map_to_type(extension?: string) {
	if (extension != undefined) {
		return type_map[extension as keyof typeof type_map] ?? 'none';
	} else {
		return 'none';
	}
}

export default function Text({ language, text, line_numbers }: { language: string, text: string, line_numbers: boolean }) {
	const lines = text.split('\n');

	const media_matcher = window.matchMedia('(prefers-color-scheme: light)');
	const [light_theme, set_light_theme] = useState(media_matcher.matches);

	function check_light_theme(event: MediaQueryListEvent) {
		if (light_theme != event.matches) {
			set_light_theme(!light_theme);
		}
	}

	useEffect(() => {
		media_matcher.addEventListener('change', check_light_theme);
		return () => {
			media_matcher.removeEventListener('change', check_light_theme);
		};
	});

	return <div className={style.group}>
		{line_numbers ? <p className={style.line_numbers}>{lines.map((_, index) => `${index}\n`)}</p> : <></>}
		<Highlight
			theme={themes[light_theme ? 'nightOwlLight' : 'oneDark']}
			code={text}
			language={language}
		>
			{({ tokens, getLineProps, getTokenProps }) => (
				<div>
					{tokens.map((line, i) => (
						<div key={i} {...getLineProps({ line })}>
							{line.map((token, key) => (
								<span key={key} {...getTokenProps({ token })} />
							))}
						</div>
					))}
				</div>
			)}
		</Highlight>
	</div>;
}
